from pathlib import Path
from random import choice
from typing import Literal

import uvicorn
from fastapi import FastAPI, Request
from fastapi.responses import HTMLResponse, JSONResponse
from fastapi.templating import Jinja2Templates

from ezquiz.ezquiz import Q


class APIGame:
    def __init__(self, title: str, qs: dict[str, Q]) -> None:
        self.title = title
        self.qs = qs
        self.score = 0
        self.mistakes = 0
        self.streak = 0
        self.history = []

    def start(
        self,
        **fastapi_kw,
    ):
        app = FastAPI(**fastapi_kw)

        templates = Jinja2Templates(directory=Path(__file__).parent / "templates")

        @app.get("/", response_class=HTMLResponse)
        async def landing_page(request: Request):
            return templates.TemplateResponse(
                "index.html",
                context={
                    "request": request,
                    "title": self.title,
                    "categories": list(self.qs.keys()),
                },
            )

        @app.post("/api/next", response_class=JSONResponse)
        async def next_question(request: Request):
            """Get the next question in the session."""
            # return a random question from selected categories
            data = await request.json()
            print(data)
            categories = data.get("categories", [])

            # Get all available questions from selected categories
            cat = choice(categories)

            q = self.qs[cat]
            seed = q.get_seed()
            prompt = q.ask(seed)

            return JSONResponse(
                {
                    "complete": False,
                    "question": {"category": cat, "seed": seed, "text": prompt},
                }
            )

        @app.post("/api/submit", response_class=JSONResponse)
        async def submit_answer(request: Request):
            data = await request.json()
            print(data)
            cat = data["category"]
            seed = data["seed"]
            submitted_ans = data["answer"]

            q = self.qs[cat]
            correct_ans = q.correct(seed)
            correct = q.check(correct_ans, submitted_ans)
            explain = q.explain(seed)

            return JSONResponse(
                {
                    "correct": correct,
                    "submitted_answer": submitted_ans,
                    "correct_answer": correct_ans,
                    "explanation": explain,
                }
            )

        uvicorn.run(app)



from typing import Literal


class CLIGame:
    def __init__(self) -> None:
        self.score = 0
        self.mistakes = 0
        self.streak = 0
        self.history = []

    def start(
        self,
        mode: Literal["timed", "score", "mistakes", "total", "streak"] = "score",
        param=10,
    ):
        pass



from typing import Callable, Generic, Literal, TypedDict, TypeVar

T = TypeVar("T")

Prompt = TypedDict("Prompt", {"type": Literal["text", "audio"], "value": str})

Explain = TypedDict("Explain", {"type": Literal["text", "text_diff"], "value": str})


class Q(Generic[T]):
    def __init__(
        self,
        get_seed: Callable[[], T],
        ask: Callable[[T], str],
        correct: Callable[[T], str],
        check: Callable[[T, str], bool] | None = None,
        explain: Callable[[T], Explain] | None = None,
    ):
        self.get_seed = get_seed
        self.ask = ask
        self.correct = correct

        if check is None:
            self.check = (
                lambda correct_ans, submitted_ans: str(correct_ans) == submitted_ans
            )
        else:
            self.check = check

        if explain is None:
            self.explain = lambda _: "{textdiff}"

        else:
            self.explain = explain



from random import randint

from ezquiz import APIGame, Q


def get_ints():
    return randint(1, 10), randint(1, 10)


def ask_addition(t):
    a, b = t
    return f"what is {a} + {b}?"


def correct_addition(t):
    return t[0] + t[1]


AddQ = Q[tuple[int, int]](
    get_seed=get_ints,
    ask=ask_addition,
    correct=correct_addition,
)


def ask_mult(t):
    a, b = t
    return f"what is {a} * {b}?"


def correct_mult(t):
    a, b = t
    return a * b


MultQ = Q[tuple[int, int]](
    get_seed=get_ints,
    ask=ask_mult,
    correct=correct_mult,
)

game = APIGame(
    "Basic Math Test",
    {
        "addition": AddQ,
        "multiplication": MultQ,
    },
)

game.start(root_path="/test")
